/*
 * $LastChangedRevision: 1791 $
 * $LastChangedDate:: 2020-01-27 17:59:05 +0100#$
 *
 * This file is part of X2C. http://x2c.lcm.at/
 *
 * Copyright (c) 2016, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
/*
 * This file is licensed according to the BSD 3-clause license as follows:
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
 *       the names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "CommonFcts.h"
#include "TableStruct.h"
#include "LNet.h"
#include "Services.h"
#include "BlockServices.h"
#include "CommonFcts.h"
#include "Hardware.h"
#include "X2C.h"
#include "Main.h"

/* X2C maintenance structure */
volatile tTableStruct TheTableStruct;
volatile tTableStruct* TableStruct = &TheTableStruct;

/* LNet protocol buffersize & node identifier */
#define LNET_BUFFERSIZE ((uint8)255)
#define LNET_NODEID ((uint8)1)

static tSerial interface;
static tLNet protocol;
static uint8 bufferLNet[LNET_BUFFERSIZE];

/* used by init version info */
const struct {
	uint8 date[11];
	uint8 time[8];
} compilationDate = { __DATE__, __TIME__ };

/* private prototypes */
static void initVersionInfo(volatile tTableStruct* tblStruct, uint16 appVersion);

/** 
 * @brief Main function.
 * 
 * @return The main function will never return due to the never ending loop.
 */
int main(void)
{
	/** - initialize system clock */
	initClock();
	
	/** - initialize "integrated bootloader":
	 *    - configuration of LNet protocol:
	 *      - Node-ID: 1
	 *      - Buffer size: 255
	 *      .
	 *    .
	 *  - initialize serial interface
	 *    - configuration of USART2:
     *      - Baudrate: 115.2kB/s
     *      - Data bits: 8
     *      - Parity: none
     *      - Stop bits: 1
	 *      .
	 *    .
	 */
    initTableStruct();
    initSerialGeneric(&interface);
    initSerial(&interface);
    initLNet(&protocol, bufferLNet, LNET_BUFFERSIZE, LNET_NODEID);
    initServiceTable((tProtocol*)&protocol);
    addCoreServices((tProtocol*)&protocol);
    addBlockServices((tProtocol*)&protocol);
    addTableStructProtocol((tProtocol*)&protocol);
    linkSerial((tProtocol*)&protocol, &interface);
	KICK_DOG;

	initVersionInfo(TableStruct, (uint16)0x0001);
	TableStruct->DSPState = PRG_LOADED_STATE;

	/** - initialize X2C */
    X2C_Init();
    KICK_DOG;

	/** - initialize hardware */
	initHardware();
	KICK_DOG;
	
	/** - never ending loop -> interrupt driven algorithm */
    while(1)
	{
		/* serial communication with X2C communicator */
		protocol.communicate((tProtocol*)&protocol);
	}
}


/** 
 * @brief Main control task.
 * 
 * Calling rate = 100us
 */
void mainTask(void)
{
	/** - assign inports */
	/* read user button */
	if (GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0) == 0)
	{
		x2cModel.inports.bB1_User = 0;
	}
	else
	{
		x2cModel.inports.bB1_User = INT16_MAX;
	}	
		
	/** - update X2C */
	X2C_Update();

	/** - update outports */
	/* blue LED control */
	if (*x2cModel.outports.bLD4 == 0)
	{
		GPIO_ResetBits(GPIOC, GPIO_Pin_8);
	}
	else
	{
		GPIO_SetBits(GPIOC, GPIO_Pin_8);
	}
	
	/* green LED control */
	if (*x2cModel.outports.bLD3 == 0)
	{
		GPIO_ResetBits(GPIOC, GPIO_Pin_9);
	}
	else
	{
		GPIO_SetBits(GPIOC, GPIO_Pin_9);
	}

	KICK_DOG;
}

/**
 * @brief Routine to set version.
 *
 *				Adds frame compilation date, time and version info to X2C maintenance structure.
 * 
 * @param tblStruct X2C maintenance structure.
 * @param appVersion Version number.
 */
static void initVersionInfo(volatile tTableStruct* tblStruct, uint16 appVersion)
{
	tblStruct->framePrgVersion = appVersion;
	tblStruct->framePrgCompDateTime = (uint8*)&compilationDate;
}
